from collections.abc import Iterable
from datetime import datetime

from .models import Pet, Person, UnattachedPerson, Driver, Car, Admin, Log


def test_relation(nrm):
    org = Car(
        name='Testing Inc',
    ).save()

    user = Driver(
        name='Sam',
    ).save()

    user.cars.set([org])
    user.save()

    assert user.cars.count() == 1

    assert user.cars.all()[0].id == org.id
    assert nrm.redis.sismember('driver:{}:srel_cars'.format(user.id), org.id)

    same = Driver.get(user.id)
    same.cars.all()

    assert same.cars.all()[0].id == org.id
    assert same.to_json(include=['cars'])['cars'][0]['name'] == 'Testing Inc'


def test_delete_cascade(nrm):
    doggo = Pet(name='doggo').save()
    catto = Pet(name='catto').save()

    owner = Person(
        name='John',
    ).save()
    owner.pets.set([doggo, catto])

    assert doggo in owner.pets
    assert catto in owner.pets

    owner.delete()

    assert Pet.get(doggo.id) is None
    assert Pet.get(catto.id) is None


def test_delete_preserves_related_no_cascade(nrm):
    doggo = Pet(name='doggo').save()
    catto = Pet(name='catto').save()

    owner = UnattachedPerson(
        name='John',
    ).save()
    owner.pets.set([doggo, catto])

    assert doggo in owner.pets
    assert catto in owner.pets

    owner.delete()

    assert Pet.get(doggo.id).name == 'doggo'
    assert Pet.get(catto.id).name == 'catto'


def test_many_to_many_relationship(nrm):
    u1 = Driver(name='u1').save()
    u2 = Driver(name='u2').save()
    o1 = Car(name='o1').save()
    o2 = Car(name='o2').save()

    # test adding a relationship creates the inverse
    u1.cars.set([o1, o2])
    u1.save()

    o1drivers = o1.drivers.all()
    o2drivers = o2.drivers.all()

    assert type(o1drivers) == list
    assert type(o2drivers) == list

    assert len(o1drivers) == 1
    assert len(o2drivers) == 1

    assert o1drivers[0].name == 'u1'
    assert o2drivers[0].name == 'u1'

    u2.cars.set([o2])

    o1drivers = o1.drivers.all()
    o2drivers = o2.drivers.all()

    assert type(o1drivers) == list
    assert type(o2drivers) == list

    assert len(o1drivers) == 1
    assert len(o2drivers) == 2

    o2drivers.sort(key=lambda x: x.name)

    assert o1drivers[0].name == 'u1'
    assert o2drivers[0].name == 'u1'
    assert o2drivers[1].name == 'u2'

    # test deleting an object deletes the relationship in the related

    u1.delete()

    o1drivers = o1.drivers.all()
    o2drivers = o2.drivers.all()

    assert type(o1drivers) == list
    assert type(o2drivers) == list

    assert len(o1drivers) == 0
    assert len(o2drivers) == 1


def test_querying_related(nrm):
    o1 = Car(name='o1').save()
    u1 = Driver(name='u1').save()
    u2 = Driver(name='u2').save()
    u1.cars.set([o1])

    assert u1 in o1.drivers
    assert u2 not in o1.drivers


def test_can_filter_related(nrm):
    p = Person(name='Juan').save()

    pets = [
        Pet(name='bc').save(),  # 0
        Pet(name='bd').save(),  # 1
        Pet(name='cd').save(),  # 2
    ]

    p.pets.set(pets)

    assert isinstance(p.pets.q().filter(name__startswith='pa'), Iterable)

    res = list(map(
        lambda x: x.id,
        p.pets.q().filter(name__startswith='b', name__endswith='d')
    ))

    assert res == [pets[1].id]

    res = list(map(
        lambda x: x.id,
        p.pets.q().filter(name__startswith='b').filter(name__endswith='d')
    ))

    assert res == [pets[1].id]


def test_foreign_key(nrm):
    owner = Person(name='John').save()
    pet = Pet(name='doggo').save()

    pet.owner.set(owner)
    ownerpets = owner.pets.all()

    assert pet.owner.get() is not None
    assert pet.owner.get().id == owner.id

    ownerpets = owner.pets.all()

    assert type(ownerpets) == list
    assert len(ownerpets) == 1
    assert ownerpets[0].id == pet.id

    pet.delete()

    ownerpets = owner.pets.all()

    assert type(ownerpets) == list
    assert len(ownerpets) == 0


def test_foreign_key_inverse(nrm):
    pet = Pet(name='doggo').save()
    owner = Person(name='John').save()
    owner.pets.set([pet])

    petowner = pet.owner.get()
    ownerpets = owner.pets.all()

    assert petowner is not None
    assert petowner.id == owner.id

    assert type(ownerpets) == list
    assert len(ownerpets) == 1
    assert ownerpets[0].id == pet.id


def test_delete_relation(nrm):
    c1 = Car().save()
    c2 = Car().save()

    d1 = Driver().save()

    d1.cars.set([c1, c2])

    assert c1 in d1.cars
    assert c2 in d1.cars
    assert d1 in c1.drivers
    assert d1 in c2.drivers

    d1.cars.remove(c1)

    assert c1 not in d1.cars
    assert c2 in d1.cars
    assert d1 not in c1.drivers
    assert d1 in c2.drivers


def test_clear(nrm):
    owner = Person(name='Juan').save()

    pets = [
        Pet(name='doggo').save(),
        Pet(name='catto').save(),
    ]

    owner.pets.set(pets)

    assert owner.pets.count() == 2

    assert pets[0].owner.get() is not None
    assert pets[1].owner.get() is not None

    owner.pets.clear()

    assert owner.pets.count() == 0

    assert pets[0].owner.get() is None
    assert pets[1].owner.get() is None


def test_get_relation(nrm):
    c1 = Car().save()
    c2 = Car().save()

    d1 = Driver().save()

    d1.cars.set([c1, c2])

    cars = d1.cars.all()

    orig = sorted([c1, c2], key=lambda c: c.id)
    cars = sorted(cars, key=lambda c: c.id)

    assert orig == cars


def test_get_foreignid_relation(nrm):
    pet = Pet().save()
    owner = Person().save()
    owner.pets.add(pet)

    assert pet.owner.get().id == owner.id

    owner.pets.remove(pet)

    assert pet.owner.get() is None


def test_sorted_set_relation(nrm):
    owner = Admin(name='Juan').save()

    logs = [
        Log(date=datetime(2020, 1, 1), data='1').save(),
        Log(date=datetime(2020, 1, 1), data='1').save(),
        Log(date=datetime(2020, 1, 1), data='1').save(),
    ]

    owner.logs.set(logs[:-1])
    owner.logs.add(logs[-1])

    assert owner.logs.count() == 3

    for old, new in zip(logs, owner.logs.all()):
        assert old.id == new.id

    for log in logs:
        assert log in owner.logs

    owner.delete()

    for log in logs:
        assert log.owner.get() is None



import json

from .models import Something


def test_create_sends_message(nrm):
    p = nrm.redis.pubsub(ignore_subscribe_messages=True)
    p.psubscribe('something', 'something:*')

    thing = Something(name='the thing', abbr='TFC').save()

    messages = p.listen()

    message = next(messages)
    assert message is not None
    assert message['channel'] == b'something'
    assert json.loads(message['data'].decode('utf8')) == {
        'event': 'create',
        'data': thing.to_json(),
    }

    message = next(messages)
    assert message is not None
    assert message['channel'] == thing.key().encode()
    assert json.loads(message['data'].decode('utf8')) == {
        'event': 'create',
        'data': thing.to_json(),
    }

    p.unsubscribe()


def test_delete_sends_message(nrm):
    thing = Something(name='the thing', abbr='TFC').save()

    p = nrm.redis.pubsub(ignore_subscribe_messages=True)
    p.psubscribe('something', 'something:*')

    thing.delete()

    messages = p.listen()

    message = next(messages)
    assert message is not None
    assert message['channel'] == b'something'
    assert json.loads(message['data'].decode('utf8')) == {
        'event': 'delete',
        'data': thing.to_json(),
    }

    message = next(messages)
    assert message is not None
    assert message['channel'] == thing.key().encode()
    assert json.loads(message['data'].decode('utf8')) == {
        'event': 'delete',
        'data': thing.to_json(),
    }

    p.unsubscribe()


def test_update_sends_message(nrm):
    thing = Something(name='the thing', abbr='TFC').save()

    p = nrm.redis.pubsub(ignore_subscribe_messages=True)
    p.psubscribe('something', 'something:*')

    thing.update(name='renamed thing')

    messages = p.listen()

    message = next(messages)
    assert message is not None
    assert message['channel'] == b'something'
    data = json.loads(message['data'].decode('utf8'))
    assert data == {
        'event': 'update',
        'data': thing.to_json(),
    }

    message = next(messages)
    assert message is not None
    assert message['channel'] == thing.key().encode()
    data = json.loads(message['data'].decode('utf8'))
    assert data == {
        'event': 'update',
        'data': thing.to_json(),
    }

    assert data['data']['name'] == 'renamed thing'

    p.unsubscribe()



from collections.abc import Iterable
from coralillo.datamodel import debyte_string
from coralillo.errors import ModelNotFoundError
from .models import House, Table, Ship, Tenanted, SideWalk, Pet
import pytest


def test_create_user(nrm):
    user = Table(
        name='John',
    ).save()

    assert user.name == 'John'

    assert nrm.redis.sismember('table:members', user.id)
    assert nrm.redis.hget('table:{}:obj'.format(user.id), 'name') == b'John'


def test_retrieve_user_by_id(nrm):
    carla = Table(name='Carla',).save()

    Table(name='Roberta',).save()

    read_user = Table.get(carla.id)

    assert read_user.name == carla.name


def test_retrieve_by_index(nrm):
    titan = Ship(name='te trece', code='T13',).save()

    Ship(name='te catorce', code='T14',).save()

    found_ship = Ship.get_by('code', 'T13')

    assert found_ship is not None
    assert titan == found_ship


def test_filter(nrm):
    with pytest.raises(AttributeError) as excinfo:
        Pet.q().filter(legs__in=(3, 4))

    assert str(excinfo.value) == 'Model Pet does not have field legs'

    with pytest.raises(AttributeError) as excinfo:
        Pet.q().filter(name__foo=True)

    assert str(excinfo.value) == 'Filter foo does not exist'

    pets = [
        Pet(name='bc').save(),  # 0
        Pet(name='bd').save(),  # 1
        Pet(name='cd').save(),  # 2
    ]

    assert isinstance(Pet.q(), Iterable)

    res = list(map(
        lambda x: x.id,
        Pet.q().filter(name__startswith='b', name__endswith='d')
    ))

    assert res == [pets[1].id]

    res = list(map(
        lambda x: x.id,
        Pet.q().filter(name__startswith='b').filter(name__endswith='d')
    ))

    assert res == [pets[1].id]

    assert Pet.q().filter(name='bd').one() == pets[1]


def test_update_keep_index(nrm):
    ship = Ship(name='the ship', code='TS').save()

    ship.update(name='updated name')

    assert ship.code == 'TS'
    assert ship.name == 'updated name'

    assert debyte_string(nrm.redis.hget('ship:index_code', 'TS')) == ship.id


def test_update_changes_index(nrm):
    ship = Ship(code='THECODE').save()

    ship.update(code='NEWCODE')

    assert ship.code == 'NEWCODE'

    assert debyte_string(nrm.redis.hget('ship:index_code', 'NEWCODE')) == ship.id
    assert nrm.redis.hget('ship:index_code', 'THECODE') is None


def test_get(nrm):
    org = Table(name='Juan').save()
    got = Table.get(org.id)

    assert org == got


def test_get_all(nrm):
    p1 = Table(name='Juan').save()
    p2 = Table(name='Pepe').save()

    allitems = Table.all()

    allitems.sort(key=lambda x: x.name)

    item1 = allitems[0]
    item2 = allitems[1]

    assert item1 == p1
    assert item2 == p2


def test_bounded_model(nrm):
    dev = Tenanted(
        name='foo',
    ).save()

    assert not nrm.redis.exists('tenanted:' + dev.id)
    assert nrm.redis.exists('testing:tenanted:{}:obj'.format(dev.id))
    assert nrm.redis.hget('testing:tenanted:{}:obj'.format(dev.id), 'name') == b'foo'
    assert nrm.redis.sismember('testing:tenanted:members', dev.id)


def test_delete(nrm):
    dev = Tenanted(
        code='foo',
    ).save()

    assert Tenanted.get(dev.id) is not None

    dev.delete()

    assert Tenanted.get(dev.id) is None
    assert not nrm.redis.sismember('testing:tenanted:members', dev.id)


def test_delete_index(nrm):
    ship = Ship(code='A12').save()

    assert debyte_string(nrm.redis.hget('ship:index_code', 'A12')) == ship.id

    ship.delete()

    assert not nrm.redis.hexists('ship:index_code', 'A12')


def test_is_object_key(nrm):
    ship = Ship(code='A12').save()

    assert Ship.is_object_key(ship.key())


def test_fqn(nrm):
    ship = Ship(code='A12').save()

    assert ship.fqn() == 'ship:{}'.format(ship.id)


def test_model_table_conversion(nrm):
    sw = SideWalk(name='foo').save()

    assert nrm.redis.exists('side_walk:members')
    assert nrm.redis.exists('side_walk:{}:obj'.format(sw.id))


def test_object_count(nrm):
    sw1 = SideWalk(name='1').save()
    assert SideWalk.count() == 1

    SideWalk(name='2').save()
    assert SideWalk.count() == 2

    sw1.delete()
    assert SideWalk.count() == 1


def test_get_or_exception(nrm):
    with pytest.raises(ModelNotFoundError):
        SideWalk.get_or_exception('nonsense')

    with pytest.raises(ModelNotFoundError):
        Ship.get_by_or_exception('code', 'nonsense')


def test_recover_none_int(nrm):
    h = House.validate()
    h.save()

    House.get(h.id)
    assert h.number is None



from coralillo import Engine, Model, fields
from coralillo.errors import UnboundModelError
from random import choice
import pytest


def test_create_engine():
    eng1 = Engine(db=11)
    eng2 = Engine(db=12)

    class Dog(Model):
        name = fields.Text()

        class Meta:
            engine = eng1

    class Cat(Model):
        name = fields.Text()

        class Meta:
            engine = eng2

    doggo = Dog(name='doggo').save()
    catto = Cat(name='catto').save()

    assert eng1.redis.exists('dog:{}:obj'.format(doggo.id))
    assert not eng1.redis.exists('cat:{}:obj'.format(catto.id))

    assert eng2.redis.exists('cat:{}:obj'.format(catto.id))
    assert not eng2.redis.exists('dog:{}:obj'.format(doggo.id))


def test_set_engine_delayed():
    class Foo(Model):
        name = fields.Text()

    with pytest.raises(UnboundModelError):
        Foo.get_engine()

    eng = Engine()

    Foo.set_engine(eng)

    assert Foo.get('de') is None


def test_unbound_models():
    class Dog(Model):
        name = fields.Text()

    with pytest.raises(UnboundModelError):
        Dog(mame='doggo').save()


def test_register_lua_script():
    eng = Engine(db=0)

    eng.lua.register('my_script', 'return ARGV[1]')

    assert eng.lua.my_script is not None
    assert eng.lua.my_script(args=[4]) == b'4'


def test_can_replace_id_function():
    def simple_ids():
        return ''.join(choice('123456789abcdef') for c in range(11))

    simple_eng = Engine(id_function=simple_ids)
    uuid_eng = Engine()

    class SimpleDog(Model):
        name = fields.Text()

        class Meta:
            engine = simple_eng

    class UuidDog(Model):
        name = fields.Text()

        class Meta:
            engine = uuid_eng

    simple_doggo = SimpleDog(name='doggo').save()
    assert len(simple_doggo.id) == 11

    uuid_doggo = UuidDog(name='doggo').save()
    assert len(uuid_doggo.id) == 32



from coralillo.datamodel import debyte_string

# these return false if the value is null
NULL_AFFECTED_FILTERS = ['lt', 'lte', 'gt', 'gte', 'startswith', 'endswith']

# these ones don't give a shit about null values
FILTERS = ['eq', 'ne'] + NULL_AFFECTED_FILTERS


class QuerySet:

    def __init__(self, cls, iterator):
        self.iterator = iterator
        self.filters = []
        self.cls = cls

    def __iter__(self):
        return self

    def __next__(self):
        for item in self.iterator:
            obj = self.cls.get(debyte_string(item))

            if self.matches_filters(obj):
                return obj

        raise StopIteration

    def matches_filters(self, item):
        for filt in self.filters:
            if not filt(item):
                return False

        return True

    def make_filter(self, fieldname, query_func, expct_value):
        ''' makes a filter that will be appliead to an object's property based
        on query_func '''

        def actual_filter(item):
            value = getattr(item, fieldname)

            if query_func in NULL_AFFECTED_FILTERS and value is None:
                return False

            if query_func == 'eq':
                return value == expct_value
            elif query_func == 'ne':
                return value != expct_value
            elif query_func == 'lt':
                return value < expct_value
            elif query_func == 'lte':
                return value <= expct_value
            elif query_func == 'gt':
                return value > expct_value
            elif query_func == 'gte':
                return value >= expct_value
            elif query_func == 'startswith':
                return value.startswith(expct_value)
            elif query_func == 'endswith':
                return value.endswith(expct_value)

        actual_filter.__doc__ = '{} {} {}'.format('val', query_func, expct_value)

        return actual_filter

    def filter(self, **kwargs):
        for key, value in kwargs.items():
            try:
                fieldname, query_func = key.split('__', 1)
            except ValueError:
                fieldname, query_func = key, 'eq'

            if not hasattr(self.cls, fieldname):
                raise AttributeError('Model {} does not have field {}'.format(
                    self.cls.__name__,
                    fieldname,
                ))

            if query_func not in FILTERS:
                raise AttributeError('Filter {} does not exist'.format(query_func))

            self.filters.append(self.make_filter(fieldname, query_func, value))

        return self

    def one(self):
        return next(self)

    def all(self):
        return list(self)



from coralillo import Engine, Model, fields
from coralillo.auth import PermissionHolder
import pytest

from .models import bound_models


@pytest.fixture
def nrm():
    nrm = Engine()

    nrm.lua.drop(args=['*'])

    bound_models(nrm)

    return nrm


@pytest.fixture
def user(nrm):
    class User(Model, PermissionHolder):
        name = fields.Text()

        class Meta:
            engine = nrm

    return User(
        name='juan',
    ).save()



from coralillo import Model, datamodel
from coralillo.hashing import check_password, make_password
from coralillo.fields import Text, Location, Hash, Bool, MissingFieldError
from coralillo.fields import Integer, InvalidFieldError, Float, Datetime, Dict
from datetime import datetime
import time
import os
import pytest

from .models import Truck, User, Subscription

os.environ['TZ'] = 'UTC'
time.tzset()


def test_field_text():
    field = Text(name='field', default='pollo')

    assert field.recover(None, {'field': 'foo'}, 'field') == 'foo'
    assert field.prepare('foo') == 'foo'
    assert field.validate(None, None, 'field') == 'pollo'
    assert field.recover(None, {'field': ''}, 'field') == ''
    assert field.recover(None, {'field': None}, 'field') is None
    assert field.recover(None, {'field': 'None'}, 'field') is None


def test_field_text_validate():
    field = Text(name='field')

    with pytest.raises(MissingFieldError):
        field.validate(None, '', None)


def test_field_hash():
    field = Hash(name='field')

    assert field.recover(None, {'field': 'foo'}, 'field') == 'foo'
    assert field.validate(None, 'foo', 'field') != 'foo'
    assert field.recover(None, {'field': None}, 'field') is None
    assert field.recover(None, {'field': 'None'}, 'field') is None


def test_field_bool():
    field = Bool(name='field')

    assert field.recover(None, {'field': 'True'}, 'field') is True
    assert field.recover(None, {'field': 'False'}, 'field') is False
    assert field.prepare(True) == 'True'
    assert field.prepare(False) == 'False'
    assert field.recover(None, {'field': None}, 'field') is None
    assert field.recover(None, {'field': 'None'}, 'field') is None

    assert field.validate(None, 'true', 'field') is True
    assert field.validate(None, 'false', 'field') is False
    assert field.validate(None, '1', 'field') is True
    assert field.validate(None, '0', 'field') is False
    assert field.validate(None, '', 'field') is False


def test_field_int():
    field = Integer(name='field')

    assert field.recover(None, {'field': '35'}, 'field') == 35
    assert field.recover(None, {'field': '0'}, 'field') == 0
    assert field.recover(None, {'field': ''}, 'field') is None
    assert field.recover(None, {'field': None}, 'field') is None
    assert field.recover(None, {'field': 'None'}, 'field') is None

    assert field.prepare(35) == '35'
    assert field.prepare(0) == '0'

    assert field.validate(None, 0, 'field') == 0
    assert field.validate(None, '0', 'field') == 0

    with pytest.raises(MissingFieldError):
        field.validate(None, '', 'field')

    with pytest.raises(InvalidFieldError):
        field.validate(None, 'pollo', 'field')


def test_field_int_no_required():
    field = Integer(name='field', required=False)

    assert field.validate(None, None, 'field') is None
    assert field.validate(None, '', 'field') is None


def test_field_float():
    field = Float(name='field')

    assert field.recover(None, {'field': '3.14'}, 'field') == 3.14
    assert field.recover(None, {'field': '0'}, 'field') == 0.0
    assert field.recover(None, {'field': ''}, 'field') is None
    assert field.recover(None, {'field': None}, 'field') is None
    assert field.recover(None, {'field': 'None'}, 'field') is None

    assert field.prepare(3.14) == '3.14'
    assert field.prepare(0.0) == '0.0'

    assert field.validate(None, 0, 'field') == 0
    assert field.validate(None, '0', 'field') == 0


def test_field_date():
    field = Datetime(name='field')

    assert field.recover(None, {'field': '1499794899'}, 'field') == datetime(2017, 7, 11, 17, 41, 39)
    assert field.recover(None, {'field': ''}, 'field') is None
    assert field.recover(None, {'field': None}, 'field') is None
    assert field.recover(None, {'field': 'None'}, 'field') is None
    assert field.prepare(datetime(2017, 7, 11, 17, 41, 39)) == '1499794899'


def test_field_location(nrm):
    class MyModel(Model):
        field = Location()

        class Meta:
            engine = nrm

    obj = MyModel(field=datamodel.Location(-103.3590, 20.7240)).save()

    assert nrm.redis.type('my_model:geo_field') == b'zset'
    assert MyModel.get(obj.id).field == datamodel.Location(-103.3590, 20.7240)


def test_field_location_validate(nrm):
    class MyModel(Model):
        field = Location()

        class Meta:
            engine = nrm

    item = MyModel.validate(field='-100,20').save()
    assert item.field == datamodel.Location(-100, 20)

    item_rec = MyModel.get(item.id)
    assert item_rec.field == datamodel.Location(-100, 20)


def test_field_location_none(nrm):
    class MyModel(Model):
        field = Location(required=False)

        class Meta:
            engine = nrm

    item = MyModel.validate().save()

    item_rec = MyModel.get(item.id)

    assert item_rec.field is None


def test_password_check():
    user = User(
        password=make_password('123456'),
    ).save()

    assert check_password('123456', user.password)
    assert user.password != '123456'


def test_empty_field_dict(nrm):
    class Dummy(Model):
        dynamic = Dict()

        class Meta:
            engine = nrm

    a = Dummy().save()
    loaded_a = Dummy.get(a.id)

    assert loaded_a.dynamic == {}


def test_field_dict(nrm):
    class Dummy(Model):
        name = Text()
        dynamic = Dict()

        class Meta:
            engine = nrm

    # Create and save models with a dict field
    a = Dummy(
        name='dummy',
        dynamic={
            '1': 'one',
        },
    ).save()

    b = Dummy(
        name='dummy',
        dynamic={
            '2': 'two',
        },
    ).save()

    c = Dummy(
        name='dummy',
        dynamic={
            '1': 'one',
            '2': 'two',
        },
    ).save()

    # override dict
    a.dynamic = {
        '3': 'thre',
    }

    a.save()

    loaded_a = Dummy.get(a.id)
    loaded_b = Dummy.get(b.id)
    loaded_c = Dummy.get(c.id)

    assert a.dynamic == loaded_a.dynamic
    assert b.dynamic == loaded_b.dynamic
    assert c.dynamic == loaded_c.dynamic


def test_field_position_delete():
    c1 = Truck(last_position=datamodel.Location(-90, 21)).save()
    c2 = Truck(last_position=datamodel.Location(-90, 22)).save()

    assert Truck.get(c1.id).last_position == datamodel.Location(-90, 21)
    assert Truck.get(c2.id).last_position == datamodel.Location(-90, 22)

    c1.delete()

    assert Truck.get(c1.id) is None

    assert Truck.get(c2.id).last_position == datamodel.Location(-90, 22)


def test_tree_index():
    Subscription(key_name='a:b:c:d').save()
    s1 = Subscription(key_name='a:b:c').save()
    s2 = Subscription(key_name='a:b').save()
    s3 = Subscription(key_name='a').save()

    assert Subscription.tree_match('key_name', 'a:b:c') == [s1, s2, s3]

    s3.delete()
    assert Subscription.tree_match('key_name', 'a:b:c') == [s1, s2]

    s2.delete()
    assert Subscription.tree_match('key_name', 'a:b:c') == [s1]

    s1.delete()
    assert Subscription.tree_match('key_name', 'a:b:c') == []



from coralillo.utils import parse_embed


def test_parse_embed():
    array = ['object']
    output = [['object', None]]
    assert parse_embed(array) == output

    array = ['object.field']
    output = [['object', ['field']]]
    assert parse_embed(array) == output

    array = ['object.field', 'foo', 'object.var']
    output = [['foo', None], ['object', ['field', 'var']]]
    assert parse_embed(array) == output
